#ifndef SCENEBASE_H
#define SCENEBASE_H

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/glew.h>
#include <GL/glut.h>
#include <string>

#include "../Mathlib.h"

class SplineGL;

// Classe de base abstraite des scnes
class SceneBase
{
public:
	// Initialisation des donnes propres  la scne
	virtual void Init() {}

	// Destruction des donnes propres  la scne
	virtual void Destroy();

	// Evolution des donnes et traitement des commandes synchones
	virtual void Idle(float fElapsedTime) {}

	// Traitement des commandes asynchones
	virtual void Keyboard(bool special, unsigned char key);

	// Dmarrage de la scne
	virtual void Reset();

	// Pr-affichage de la scne (peut utiliser des FBO)
	virtual void PreRender() {}

	// Affichage de la scne (ne peut pas utiliser des FBO)
	virtual void Render() = 0;

	// On met  jour la position dans les splines de camra
	void InterpolCameraTraj(float fElapsedTime);

	// Affiche la trajectoire de la camra
	void DrawTraj();	

	SceneBase();
	virtual ~SceneBase()  {Destroy();}

private:
	SceneBase(const SceneBase& sc) {}
	SceneBase &operator=(const SceneBase& sc) {return (*this);}

public:
	// Affiche un rpre
	static void DrawAxes();

	// Affiche une chaine de caractre  l'cran
	static void DrawString(vec2 pos, void* font, const std::string& str);

	// Charge la trajectoire de la camra depuis un fichier
	bool LoadCameraTrajFromFile(const std::string& name);

	// Sauve la trajectoire de la camra dans un fichier
	bool SaveCameraTrajInFile(const std::string& name);

	inline SplineGL*	getCamEyeSpline()	const	{return m_pCamEyeTraj;}

protected:
	SplineGL*		m_pCamEyeTraj;
	SplineGL*		m_pCamLookAtTraj;
};

#endif


